<?php
/*
 * Copyright (C) 2010 - PRiSE
 * This class is based on the php library crypt.php developed by RedIRIS
 *
 * This	free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

// Constant for encrypt and decrypt data with openssl
define('PADDINGSIZE', 11);

class Crypt_RSA {

	public static function encrypt_base64($in, $key, $key_bits = 0) {		
		// Get the byte size of data string 
		$inputSize = strlen($in);

		// Get details of the key
		$res = openssl_get_privatekey($key);
		if ($key_bits==0) {
			$key_details = openssl_pkey_get_details($res);
		}
		else {
			$key_details=array('bits' => $key_bits);
		}

		// Get the output block maximun size in Bytes
		$outputBlockSize = 	$key_details['bits']/8;
		
		// Total number of blocks
		$inputBlockSize = $outputBlockSize - PADDINGSIZE;
		$numBlocks = ceil($inputSize/$inputBlockSize);
	
		// Start to encrypt.
		$blockCount = 0;
		$cryptBuffer = array();

		while ($blockCount < $numBlocks){
			$index = $blockCount * $inputBlockSize;
			$block = substr($in, $index, $inputBlockSize);
			openssl_private_encrypt($block, $crypttext, $key);
			$cryptBuffer[$blockCount] = $crypttext;		
			$blockCount++;
		}
		// Now joint the array with the blocks string encripted
		$cryptData = join("", $cryptBuffer);
	
		$base64CryptData = base64_encode($cryptData);
	
		// Return the encrypted, joined and base64 encode data string.
		return $base64CryptData;	
	}

	public static function decrypt_base64($in, $key, $error_log, $key_bits = 0) {
		// Decode the base64 input string
		$in = base64_decode($in);
	
		// Get the byte size of data string 
		$inputSize = strlen($in);

		// Get details of the key
		$res = openssl_get_publickey($key);	
		if ($key_bits==0) {
			$key_details = openssl_pkey_get_details($res);
		}
		else {
			$key_details=array('bits' => $key_bits);
		}
	
		// Get the output block maximun size in Bytes
		$outputBlockSize = 	$key_details['bits']/8;
	
		$inputBlockSize = $outputBlockSize;
		$numBlocks = ceil($inputSize/$inputBlockSize);

		// Start to decrypt.
		$blockCount = 0;
		$decryptBuffer = array();

		while ($blockCount < $numBlocks){
			$index = $blockCount * $inputBlockSize;
			$block = substr($in, $index, $inputBlockSize);
			// Decrypt the text 
			if (!openssl_public_decrypt($block, $decrypttext, $key)) {
				$msg = "Crypt_RSA::decrypt_base64(): Cannot decrypt response, check GPoA public key.";
				Log::info($msg);
				throw new Exception($msg);
			}  
			$decryptBuffer[$blockCount] = $decrypttext;		
			$blockCount++;
		}

		// Now joint the array with the blocks string encripted
		$decryptData = join("",$decryptBuffer);
	
		// Return the base64 dencode, decrypted and joined data string. 	
		return $decryptData;
	}
	
}

?>
