<?php
/*
 * Copyright (C) 2010 - PRiSE
 *
 * This	free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

class GPoAHandler {

	private $list_as;
	private $always_wayf;
	private $filter_attrs;
	private $gpoa_url;
	private $gpoa_id;
	private $gpoa_privkey;
	private $format_assertion;
        private $lcook_TTL;

	public function processConfig($config) {
		$this->always_wayf = Reader::readParameter($config['ALWAYS_WAYF']);
		$this->list_as = Reader::readParameter($config['LIST_AS']);
		$this->filter_attrs = Reader::readParameter($config['FILTER_ATTRS']);
		$this->gpoa_privkey = Reader::readParameter($config['PRIVATE_KEY_GPOA']);
		$this->gpoa_id = Reader::readParameter($config['ID_GPOA']);
		$this->lcook_TTL = Reader::readParameter($config['LCOOK']['L_TTL']);
		
		$this->gpoa_url = Reader::readParameter($config['URL_GPOA']);
		if ($this->gpoa_url == '') {
			$s = empty($_SERVER["HTTPS"]) ? '' : ($_SERVER["HTTPS"] == "on") ? "s" : ""; 
		    $temp = explode("/",strtolower($_SERVER["SERVER_PROTOCOL"]));
		    $protocol = $temp[0].$s;
		    $port = ($_SERVER["SERVER_PORT"] == "80") ? "" : (":".$_SERVER["SERVER_PORT"]); 
			$this->gpoa_url = $protocol."://".$_SERVER['SERVER_NAME'].$port.$_SERVER['SCRIPT_NAME']; 
		}
		
		$this->setDefaultFormatAssertion();
		foreach (Reader::readParameter($config['FORMAT_ATTR']) as $key => $value) {
			$this->format_assertion[$key] = $value;
		}
	}
	
	public function isCHECKRequest($params) {
		return array_key_exists("ACTION",$params)
			&& strcmp($params["ACTION"],"CHECK")==0;
	}	

	public function isATTREQResponse($params) {
		return array_key_exists("ACTION",$params)
			&& strcmp($params["ACTION"],"CHECKED")==0;
	}
	
	public function getATTREQRequest($msg_id, $as_id = '', $poa_url = '') {
		if (count($this->list_as)==0) {
			$msg = "ERROR! There isn't any AS configured";
			Log::info($msg);
			throw new Exception($msg);
		}
	
		if ($as_id == '') {
			$temp = array_keys($this->list_as);
			$as_id = $temp[0];
		}
		
		$res = $this->list_as[$as_id]['url'].
				"?ATTREQ=".urlencode($this->gpoa_id).
				"&PAPIPOAREF=".urlencode($msg_id).
				"&PAPIPOAURL=".urlencode($this->gpoa_url);
				
		if ($poa_url != '') {
			$res .= "&PAPIOPOA=".urlencode($poa_url);
		}
		
		return $res;
	}
	
	public function processATTREQResponse($params) {
		$param_as = $params["AS"];
		$param_action = $params["ACTION"];
		$param_data = $params["DATA"];
		
		$res = "error";
		if (!array_key_exists($param_as, $this->list_as)) {
			$msg = "ERROR! The AS '".$param_as."' is not valid in this GPoA.";
			Log::info($msg);
			throw new Exception($msg);
		}
		$safe = Crypt_RSA::decrypt_base64($param_data, Reader::readParameter($this->list_as[$param_as]['pubkey']), "");
		Log::debug("ATTREQ DATA decrypted: ".$safe);
	
		$temp = explode(":",$safe);
	
		$key = array_pop($temp);
		$issuedTime = array_pop($temp);
		$expiryTime = array_pop($temp);
		$assertion = implode(":",$temp);
		
		$temp = explode("@",$assertion);
		$asID = array_pop($temp);
		$attrs = implode("@",$temp);
			
		if (strcmp($asID, $param_as)!=0) {
			$msg = "ERROR! The assertion was issued by the AS '".$asID."' and the ATTREQ response by the AS '".$param_as."'.";
			Log::info($msg);
			throw new Exception($msg);
		}
		
		$currentTime = time();
		
		if ($currentTime >= $expiryTime) {
			$msg = "ERROR! The ATTREQ response message has expired. The expiry time of the assertion is ".$expiryTime." and the current time is ".$currentTime.".";
			Log::info($msg);
			throw new Exception($msg);
		}

		$old_params = $this->loadRequest($key);
			
		return array('key' => $key,
					 'issued_time' => $issuedTime,
					 'expiry_time' => $expiryTime,
					 'assertion' => $assertion,
					 'old_params' => $old_params);
	}
	
	public function getCHECKEDResponse($url_poa, $assertion, $sent_data_param) {
		$poas = array_keys($this->filter_attrs);
		
		$attrs = '';
		
		foreach ($poas as $poa_url) {
			$expreg = "/".$poa_url."/";
			$matches = array();
			preg_match($expreg, $url_poa, $matches);
			if (count($matches)>0) {
				if (!is_array($attrs)) {
					$attrs = array();
				}
				$attrs = array_merge($attrs, $this->filter_attrs[$poa_url]);
			}
		}
		
		if (is_array($attrs)) {
			$assertion = $this->filterAttributes($assertion, $attrs);
		}
		
		$now = time();
		$expiry = $now + $this->lcook_TTL;
	
		$raw = $assertion.":".$expiry.":".$now.":".$sent_data_param;
		
		$ciphered = Crypt_RSA::encrypt_base64($raw, $this->gpoa_privkey);
		
	    $join = "?";
    	if (strpos($url_poa,"?")!==false) {
            $join = "&";
	    } 
	    $res = $url_poa.$join.
				"ACTION=CHECKED".
				"&DATA=".urlencode($ciphered);
				
		return $res;
	}
	
	public function filterAttributes($assertion, $list_attr) {
		$res = array();
		
		$attrs = explode($this->format_assertion['attr_sep'], $assertion);
		
		foreach ($attrs as $attr) {
			$data_attr = explode($this->format_assertion['value_attr_sep'], $attr);
			
			if (in_array($data_attr[0], $list_attr)) {
				$res[] = $attr;
			}
		}
		
		return implode($this->format_assertion['attr_sep'], $res);
	}
	
	public function isNeededWayf() {
		return (count($this->list_as)>1 || $this->always_wayf == true);
	}
	
	public function getASFromWAYF($params) {
		$res = '';
		
		if (array_key_exists("ACTION",$params) && strcmp($params["ACTION"],"WAYF")==0) {
			$res = $params['as'];
		}
		
		return $res;
	}
	
	public function getGPoAURL() {
		return $this->gpoa_url;
	}
	
	public function generateUniqueID() {
		$id = sha1(uniqid("",true));
		
		return $id;
	}
	
	public function saveRequest($params) {
		$id = $this->generateUniqueID();
		Log::debug("Saving request ID '".$id."'");

		if (!isset($_SESSION)) {
			session_start();
		}
		$_SESSION["save_request_".$id] = $params;
		
		return $id;
	}
	
	public function loadRequest($id) {
		if (!isset($_SESSION)) {
			session_start();
		}
		Log::debug("Loading request ID '".$id."'");
		if (!array_key_exists("save_request_".$id, $_SESSION)) {
			$msg = "ERROR! Request ID '".$id."' not found";
			Log::info($msg);
			throw new Exception($msg);
		}
		return $_SESSION["save_request_".$id];
	}
	
	private function setDefaultFormatAssertion() {
		$this->format_assertion = array(
			'attr_sep' => ',',
			'value_attr_sep' => '=',
			'multivalue_attr_sep' => '|',
		);
	}
	
}

?>
